// Lab 7: Fl_File_Chooser
// CMainWindow.cpp
#include "CMainWindow.h"

#define DEFAULT_JPEG_FILE_PATH "../../JPEG/sunset.JPG"

CMainWindow::CMainWindow()
    : jpeg_image( nullptr ), file_chooser( nullptr )
{
}

void CMainWindow::show()
{
    // Window callbacks:
    win_app->callback( (Fl_Callback*)CMainWindow::callback_window_closing, (void*)(this) );
    btn_show_jpeg->callback( (Fl_Callback*)CMainWindow::callback_btn_show_jpeg, (void*)(this) );

    // Make the window visible:
    win_app->show();
}

// callback functions
void CMainWindow::callback_window_closing(Fl_Widget* widg, void* userdata_) {
    std::cout << "X button clicked -- exiting the program\n";
    while( Fl::first_window() ) {
        Fl::first_window()->hide();
    }
}

void CMainWindow::callback_btn_show_jpeg(Fl_Widget* widg, void* userdata_)
{
    CMainWindow* win = static_cast< CMainWindow* > ( userdata_ );
    win->click_btn_show_jpeg();
}

void CMainWindow::click_btn_show_jpeg()
{
    std::cout << "btn_show_jpeg click\n";
    // select file
    if ( !file_chooser ) {
        file_chooser = new Fl_File_Chooser(
            nullptr, // pathname file_chooser->directory( pathname or nullptr ) nullptr uses current directory
            "", // pattern file_chooser->filter( filter or nullptr )
            Fl_File_Chooser::SINGLE, // selection type file_chooser->type( filter or nullptr )
            "" // title
            );
    }

    // Note: file_chooser->type() arguments can be:
    // Fl_File_Chooser::SINGLE - allows the user to select a single, existing file.
    // Fl_File_Chooser::MULTI - allows the user to select one or more existing files.
    // Fl_File_Chooser::CREATE - allows the user to select a single, existing file or specify a new filename.
    // Fl_File_Chooser::DIRECTORY - allows the user to select a single, existing directory.
    file_chooser->show();

    // Hang on until the user makes a choice
    while( file_chooser->shown() ) {
        Fl::wait();
    }

    std::cout << "----------file chooser results----------\n";
    const char* file_path = DEFAULT_JPEG_FILE_PATH;
    if ( file_chooser->value() != nullptr ) {
        file_path = file_chooser->value();
        std::cout << "DIRECTORY: " << file_chooser->directory() << '\n';
        std::cout << "    VALUE: " << file_chooser->value() << '\n';
        std::cout << "    COUNT: " << file_chooser->count() << '\n';
        if ( file_chooser->count() > 1 ) {
            for ( int idx = 1; idx <= file_chooser->count(); ++idx ) {
                std::cout << " VALUE(" << idx << "): '" << file_chooser->value( idx ) << '\n';
            }
        }
    } else {
        std::cout << "The user cancelled file selection\n";
        return;
    }
    std::cout << "----------------------------------------\n";

    Fl_JPEG_Image* jpeg_image_orig = new Fl_JPEG_Image( file_path );

    // copy allow us to resize the image:
    if ( jpeg_image ) {
        //The image should be deleted when you are done with it: 
        delete jpeg_image;
    }
    jpeg_image = jpeg_image_orig->copy( box_jpeg_image->w(), box_jpeg_image->h() );
    delete jpeg_image_orig;
    box_jpeg_image->image( jpeg_image );
    win_app->redraw();
}

/*Sample output:
----------file chooser results----------
DIRECTORY: C:/igor/BCC/c255labs/labs/c255_lab07/JPEG
    VALUE: C:/igor/BCC/c255labs/labs/c255_lab07/JPEG/sunset.JPG
    COUNT: 1
----------------------------------------
*/